  PROGRAM test1
  USE blas_90
  INTEGER, PARAMETER :: wp = KIND(0.0d0)
  INTEGER, PARAMETER :: n = 4, m = 3
  REAL(wp), PARAMETER :: vec(14) = (/ 1.0_wp, -2.5_wp, -2.0_wp, 3.0_wp, &
   2.0_wp, .5_wp, -4.0_wp, 1.5_wp, 2.5_wp, -3.5_wp, 1.0_wp, -3.0_wp, &
   -2.5_wp, -2.0_wp /)
  COMPLEX(wp), PARAMETER :: zvec(8) = (/ (1.0_wp, -2.5_wp), (-2.0_wp, 3.0_wp), &
   (2.0_wp, 0.0_wp), (1.5_wp, 2.0_wp), (-.5_wp, 3.0_wp), (2.0_wp, 0.0_wp), &
   (.5_wp, 1.0_wp), (1.0_wp, 1.0_wp) /)
  REAL(wp) :: a(n,n), b(n,n), c(n,n), c_original(n,n)
  REAL(wp) :: u(n), v(n), w(n), w_original(n)
  REAL(wp) :: alpha, beta, one
  COMPLEX(wp) :: z1(m,m), z2(m,m), z(m,m), z_original(m,m)
  COMPLEX(wp) :: zu(m), zv(m), zw(m), zw_original(m)
  COMPLEX(WP) :: c1, c2, zone, ztwo

  ! REAL data

  DO i = 1, n
   a(:,i) = vec (i+1:i+n)
   b(:,i) = vec (i:i+2*(n-1):2)
   c(:,i) = vec (i+1:i+n)
  END DO
  c_original = c

  u = vec (14:14-2*(n-1):-2)
  v = vec (1:1+3*(n-1):3)
  w = vec ((/ 4, 7, 9, 5 /))
  w_original = w

  alpha = 2.5_wp
  beta = 2.0_wp
  one = 1.0_wp

  CALL write_real_mat (a,'Matrix  a')
  CALL write_real_mat (b,'Matrix  b')
  CALL write_real_mat (c,'Matrix  c')
  write(*,'(/a,2x,8f9.3)') ' vector u ', u
  write(*,'(a,2x,8f9.3)') ' vector v ', v
  write(*,'(a,2x,8f9.3)') ' vector w ', w

  ! COMPLEX data

  DO i = 1, m
   z1(:,i) = zvec (i+1:i+m)
   z2(:,i) = zvec (i:i+2*(m-1):2)
   z(:,i) = zvec (i+1:i+m)
  END DO
  z_original = z

  zu = zvec (8:8-2*(m-1):-2)
  zv = zvec (1:1+3*(m-1):3)
  zw = zvec ((/ 4, 8, 2 /))
  zw_original = zw

  c1 = zvec(5)
  c2 = zvec(7)
  zone = 1.0_wp
  ztwo = 2.0_wp

  CALL write_cmplx_mat (z1,'Matrix  z1')
  CALL write_cmplx_mat (z2,'Matrix  z2')
  CALL write_cmplx_mat (z,'Matrix  z')
  write(*,'(/a,2x,3(" (",f9.3,", ",f9.3,")"))') ' vector zu ', zu
  write(*,'(a,2x,3(" (",f9.3,", ",f9.3,")"))') ' vector zv ', zv
  write(*,'(a,2x,3(" (",f9.3,", ",f9.3,")"))') ' vector zw ', zw

  ! REAL calls

  ! c <- a b + c
  CALL  gemm (a,b,one,c)
  CALL write_real_mat (c,'a b + c', &
   'USING  CALL gemm (a,b,one,c)')

  ! c <- a b + c using key words
  c = c_original
  CALL  gemm (a=a,b=b,c=c,beta=one)
  CALL write_real_mat (c,'a b + c', &
   'USING  CALL gemm (a=a,b=b,c=c,beta=one)')

  ! c <- 2.0 a b(t)
  CALL  gemm (2.0_wp,a,blas_trans,b,c)
  CALL write_real_mat (c,'2.0 a b(t)', &
   'USING  CALL gemm (2.0_wp,a,blas_trans,b,c)')

  ! c <- 2.0 a b(t)  using key words
  CALL  gemm (c=c,a=a,alpha=2.0_wp,op_b=blas_trans,b=b)
  CALL write_real_mat (c,'2.0 a b(t)', &
   'USING  CALL gemm (c=c,a=a,alpha=2.0_wp,op_b=blas_trans,b=b)')
  
  ! c <- alpha u v(t) + c
  c = c_original
  CALL  gemm (alpha,u,v,one,c)
  CALL write_real_mat (c,'alpha u v(t) + c', &
   'USING  CALL gemm (alpha,u,v,one,c)')
  
  ! c <- alpha u v(t) + c  using key words
  c = c_original
  CALL  gemm (alpha=alpha,a=u,b=v,beta=one,c=c)
  CALL write_real_mat (c,'alpha u v(t) + c', &
   'USING  CALL gemm (alpha=alpha,a=u,b=v,beta=one,c=c)')

  ! w <- 3.0 a(t) * u + 2.0 w
  CALL  gemm (3.0_wp,blas_trans,a,u,beta,w)
  write(*,'(/a/a)') '   3.0 a(t) * u + 2.0 w', &
   '     USING  CALL  gemm (3.0_wp,blas_trans,a,u,beta,w)'
  write(*,'(2x,8f9.3)') w

  ! w <- 3.0 a(t) * u + w
  w = w_original
  CALL  gemm (alpha=3.0_wp,op_a=blas_trans,a=a,b=u,beta=beta,c=w)
  write(*,'(/a/a)') '   3.0 a(t) * u + 2.0 w', &
   '     USING  CALL  gemm (alpha=3.0_wp,op_a=blas_trans,a=a,b=u,beta=beta,c=w)'
  write(*,'(2x,8f9.3)') w

  ! COMPLEX calls

  ! z <- z1 z2(h) + c1 z
  CALL  gemm (z1,blas_conj_trans,z2,c1,z)
  CALL write_cmplx_mat(z,'z1 z2(h) + c1 z', &
   'USING  CALL  gemm (z1,blas_conj_trans,z2,c1,z)')

  ! z <- z1 z2(h) + c1 z     using key words
  z = z_original
  CALL  gemm (a=z1,b=z2,c=z,op_b=blas_conj_trans,beta=c1)
  CALL write_cmplx_mat(z,'z1 z2(h) + c1 z', &
   'USING  CALL  gemm (a=z1,b=z2,c=z,op_b=blas_conj_trans,beta=c1)')

  ! z <- c1 z1(h) z2(t) + c2 z
  z = z_original
  CALL  gemm (c1,blas_conj_trans,z1,blas_trans,z2,c2,z)
  CALL write_cmplx_mat(z,'c1 z1(h) z2(t) + c2 z', &
   'USING  CALL  gemm (c1,blas_conj_trans,z1,blas_trans,z2,c2,z)')

  ! z <- c1 z1(h) z2(t) + c2 z     using key words
  z = z_original
  CALL  gemm (a=z1,b=z2,c=z,op_a=blas_conj_trans,alpha=c1,op_b=blas_trans,beta=c2)
  CALL write_cmplx_mat(z,'c1 z1(h) z2(t) + c2 z', &
   'USING  CALL  gemm (a=z1,b=z2,c=z,op_a=blas_conj_trans,alpha=c1,op_b=blas_trans,beta=c2)')
 
  ! z <- 2.0 zu zv(h) + z
  z = z_original
  CALL  gemm (ztwo,zu,blas_conj_trans,zv,zone,z)
  CALL write_cmplx_mat (z,'2.0 zu zv(h) + z', &
   'USING  CALL gemm (ztwo,zu,blas_conj_trans,zv,zone,z)')

  ! z <- alpha zu zv(h) + z  using key words
  z = z_original
  CALL  gemm (alpha=ztwo,a=zu,op_b=blas_conj_trans,b=zv,beta=zone,c=z)
  CALL write_cmplx_mat (z,'2.0 zu zv(h) + z', &
   'USING  CALL gemm (alpha=ztwo,a=zu,op_b=blas_conj_trans,b=zv,beta=zone,c=z)')

  ! zw <- z1(t) * zu + zw
  CALL  gemm (blas_trans,z1,zu,zone,zw)
  write(*,'(/a/a)') '   z1(t) * zu + zw', &
   '     USING  CALL  gemm (blas_trans,z1,zu,zone,zw)'
  write(*,'(3(" (",f9.3,", ",f9.3,")"))') zw

  ! zw <- z1(t) * zu + zw     using key words
  zw = zw_original
  CALL  gemm (op_a=blas_trans,a=z1,b=zu,beta=zone,c=zw)
  write(*,'(/a/a)') '   z1(t) * zu + zw', &
   '     USING  CALL  gemm (op_a=blas_trans,a=z1,b=zu,beta=zone,c=zw)'
  write(*,'(3(" (",f9.3,", ",f9.3,")"))') zw

  CONTAINS
    SUBROUTINE write_real_mat (a,title,line2)
      REAL(wp), INTENT(IN) :: a(:,:)
      CHARACTER(*), INTENT(IN) :: title
      CHARACTER(*), INTENT(IN), OPTIONAL :: line2
      INTEGER :: i
      write (*,'(/,5x,a)') title
      IF (PRESENT(line2)) write (*,'(8x,a)') line2
      do i = 1, SIZE(a,1)
        write(*,'(8f9.3)') a(i,:)
      end do
    end SUBROUTINE write_real_mat
    SUBROUTINE write_cmplx_mat (a,title,line2)
      COMPLEX(wp), INTENT(IN) :: a(:,:)
      CHARACTER(*), INTENT(IN) :: title
      CHARACTER(*), INTENT(IN), OPTIONAL :: line2
      INTEGER :: i
      write (*,'(/,5x,a)') title
      IF (PRESENT(line2)) write (*,'(8x,a)') line2
      do i = 1, SIZE(a,1)
        write(*,'(3(" (",f9.3,", ",f9.3,")"))') a(i,:)
      end do
    end SUBROUTINE write_cmplx_mat
  END PROGRAM test1
