        ! 
        ! Replaces ZGEMM  36 procedures
        ! Replaces DGEMM  36 procedures (only 16 are needed)
        !
        !
        ! Level 3 COMPLEX  (ZGEMM)
        !
        !         alpha    op_a    op_b    beta    c
        !
        !  301     any    Trans.  Trans.   any     c < alpha a(T) b(T) + beta c
        !  302     any    Trans.  Trans.   0.0     c < alpha a(T) b(T)
        ! *303     any    Trans.  Herm.    any     c < alpha a(T) b(H) + beta c
        ! *304     any    Trans.  Herm.    0.0     c < alpha a(T) b(H)
        !  305     any    Trans.           any     c < alpha a(T) b + beta c
        !  306     any    Trans.           0.0     c < alpha a(T) b
        ! *307     any    Herm.   Trans.   any     c < alpha a(H) b(T) + beta c
        ! *308     any    Herm.   Trans.   0.0     c < alpha a(H) b(T)
        ! *309     any    Herm.   Herm.    any     c < alpha a(H) b(H) + beta c
        ! *310     any    Herm.   Herm.    0.0     c < alpha a(H) b(H)
        ! *311     any    Herm.            any     c < alpha a(H) b + beta c
        ! *312     any    Herm.            0.0     c < alpha a(H) b
        !  313     any            Trans.   any     c < alpha a b(T) + beta c
        !  314     any            Trans.   0.0     c < alpha a b(T)
        ! *315     any            Herm.    any     c < alpha a b(H) + beta c
        ! *316     any            Herm.    0.0     c < alpha a b(H)
        !  317     any                     any     c < alpha a b + beta c
        !  318     any                     0.0     c < alpha a b
        !  319     1.0    Trans.  Trans.   any     c < a(T) b(T) + beta c
        !  320     1.0    Trans.  Trans.   0.0     c < a(T) b(T)
        ! *321     1.0    Trans.  Herm.    any     c < a(T) b(H) + beta c
        ! *322     1.0    Trans.  Herm.    0.0     c < a(T) b(H)
        !  323     1.0    Trans.           any     c < a(T) b + beta c
        !  324     1.0    Trans.           0.0     c < a(T) b
        ! *325     1.0    Herm.   Trans.   any     c < a(H) b(T) + beta c
        ! *326     1.0    Herm.   Trans.   0.0     c < a(H) b(T)
        ! *327     1.0    Herm.   Herm.    any     c < a(H) b(H) + beta c
        ! *328     1.0    Herm.   Herm.    0.0     c < a(H) b(H)
        ! *329     1.0    Herm.            any     c < a(H) b + beta c
        ! *330     1.0    Herm.            0.0     c < a(H) b
        !  331     1.0            Trans.   any     c < a b(T) + beta c
        !  332     1.0            Trans.   0.0     c < a b(T)
        ! *333     1.0            Herm.    any     c < a b(H) + beta c
        ! *334     1.0            Herm.    0.0     c < a b(H)
        !  335     1.0                     any     c < a b + beta c
        !  336     1.0                     0.0     c < a b
        
        ! * means not needed for the REAL case
        !  all procedures with op_a or op_b of type "blas_conj_trans"
        !  are not needed but have been added for consistency
        !  with the complex case
        !  (303, 304, 307, 308, 309, 310, 311, 312, 315, 316,
        !   321, 322, 325, 326, 327, 328, 329, 330, 333, 334)
        !
        !
        ! Some of the operations could be replaced by Fortran 90 Intrinsics
        !  For example   336  ==  MATMUL(a,b)
        !                318  ==  alpha * MATMUL(a,b)
        !
        MODULE zgemm_01
        CONTAINS
        SUBROUTINE zgemm_301(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE zgemm_301
        END MODULE zgemm_01

        MODULE zgemm_02
        CONTAINS
        SUBROUTINE zgemm_302(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE zgemm_302
        END MODULE zgemm_02

        MODULE zgemm_03
        CONTAINS
        SUBROUTINE zgemm_303(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),CONJG(TRANSPOSE(b))) + beta * c
        END SUBROUTINE zgemm_303
        END MODULE zgemm_03

        MODULE zgemm_04
        CONTAINS
        SUBROUTINE zgemm_304(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),CONJG(TRANSPOSE(b)))
        END SUBROUTINE zgemm_304
        END MODULE zgemm_04

        MODULE zgemm_05
        CONTAINS
        SUBROUTINE zgemm_305(alpha,op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),b) + beta * c
        END SUBROUTINE zgemm_305
        END MODULE zgemm_05

        MODULE zgemm_06
        CONTAINS
        SUBROUTINE zgemm_306(alpha,op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),b)
        END SUBROUTINE zgemm_306
        END MODULE zgemm_06

        MODULE zgemm_07
        CONTAINS
        SUBROUTINE zgemm_307(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(CONJG(TRANSPOSE(a)),TRANSPOSE(b)) + beta * c
        END SUBROUTINE zgemm_307
        END MODULE zgemm_07

        MODULE zgemm_08
        CONTAINS
        SUBROUTINE zgemm_308(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(CONJG(TRANSPOSE(a)),TRANSPOSE(b))
        END SUBROUTINE zgemm_308
        END MODULE zgemm_08

        MODULE zgemm_09
        CONTAINS
        SUBROUTINE zgemm_309(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(CONJG(TRANSPOSE(a)),CONJG(TRANSPOSE(b))) + beta * c
        END SUBROUTINE zgemm_309
        END MODULE zgemm_09

        MODULE zgemm_10
        CONTAINS
        SUBROUTINE zgemm_310(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(CONJG(TRANSPOSE(a)),CONJG(TRANSPOSE(b)))
        END SUBROUTINE zgemm_310
        END MODULE zgemm_10

        MODULE zgemm_11
        CONTAINS
        SUBROUTINE zgemm_311(alpha,op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(CONJG(TRANSPOSE(a)),b) + beta * c
        END SUBROUTINE zgemm_311
        END MODULE zgemm_11

        MODULE zgemm_12
        CONTAINS
        SUBROUTINE zgemm_312(alpha,op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(CONJG(TRANSPOSE(a)),b)
        END SUBROUTINE zgemm_312
        END MODULE zgemm_12

        MODULE zgemm_13
        CONTAINS
        SUBROUTINE zgemm_313(alpha,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,TRANSPOSE(b)) + beta * c
        END SUBROUTINE zgemm_313
        END MODULE zgemm_13

        MODULE zgemm_14
        CONTAINS
        SUBROUTINE zgemm_314(alpha,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,TRANSPOSE(b))
        END SUBROUTINE zgemm_314
        END MODULE zgemm_14

        MODULE zgemm_15
        CONTAINS
        SUBROUTINE zgemm_315(alpha,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,CONJG(TRANSPOSE(b))) + beta * c
        END SUBROUTINE zgemm_315
        END MODULE zgemm_15

        MODULE zgemm_16
        CONTAINS
        SUBROUTINE zgemm_316(alpha,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,CONJG(TRANSPOSE(b)))
        END SUBROUTINE zgemm_316
        END MODULE zgemm_16

        MODULE zgemm_17
        CONTAINS
        SUBROUTINE zgemm_317(alpha,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,b) + beta * c
        END SUBROUTINE zgemm_317
        END MODULE zgemm_17

        MODULE zgemm_18
        CONTAINS
        SUBROUTINE zgemm_318(alpha,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,b)
        END SUBROUTINE zgemm_318
        END MODULE zgemm_18

        MODULE zgemm_19
        CONTAINS
        SUBROUTINE zgemm_319(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE zgemm_319
        END MODULE zgemm_19

        MODULE zgemm_20
        CONTAINS
        SUBROUTINE zgemm_320(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE zgemm_320
        END MODULE zgemm_20

        MODULE zgemm_21
        CONTAINS
        SUBROUTINE zgemm_321(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),CONJG(TRANSPOSE(b))) + beta * c
        END SUBROUTINE zgemm_321
        END MODULE zgemm_21

        MODULE zgemm_22
        CONTAINS
        SUBROUTINE zgemm_322(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),CONJG(TRANSPOSE(b)))
        END SUBROUTINE zgemm_322
        END MODULE zgemm_22

        MODULE zgemm_23
        CONTAINS
        SUBROUTINE zgemm_323(op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),b) + beta * c
        END SUBROUTINE zgemm_323
        END MODULE zgemm_23

        MODULE zgemm_24
        CONTAINS
        SUBROUTINE zgemm_324(op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),b)
        END SUBROUTINE zgemm_324
        END MODULE zgemm_24

        MODULE zgemm_25
        CONTAINS
        SUBROUTINE zgemm_325(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(CONJG(TRANSPOSE(a)),TRANSPOSE(b)) + beta * c
        END SUBROUTINE zgemm_325
        END MODULE zgemm_25

        MODULE zgemm_26
        CONTAINS
        SUBROUTINE zgemm_326(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(CONJG(TRANSPOSE(a)),TRANSPOSE(b))
        END SUBROUTINE zgemm_326
        END MODULE zgemm_26

        MODULE zgemm_27
        CONTAINS
        SUBROUTINE zgemm_327(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(CONJG(TRANSPOSE(a)),CONJG(TRANSPOSE(b))) + beta * c
        END SUBROUTINE zgemm_327
        END MODULE zgemm_27

        MODULE zgemm_28
        CONTAINS
        SUBROUTINE zgemm_328(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(CONJG(TRANSPOSE(a)),CONJG(TRANSPOSE(b)))
        END SUBROUTINE zgemm_328
        END MODULE zgemm_28

        MODULE zgemm_29
        CONTAINS
        SUBROUTINE zgemm_329(op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(CONJG(TRANSPOSE(a)),b) + beta * c
        END SUBROUTINE zgemm_329
        END MODULE zgemm_29

        MODULE zgemm_30
        CONTAINS
        SUBROUTINE zgemm_330(op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(CONJG(TRANSPOSE(a)),b)
        END SUBROUTINE zgemm_330
        END MODULE zgemm_30

        MODULE zgemm_31
        CONTAINS
        SUBROUTINE zgemm_331(a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,TRANSPOSE(b)) + beta * c
        END SUBROUTINE zgemm_331
        END MODULE zgemm_31

        MODULE zgemm_32
        CONTAINS
        SUBROUTINE zgemm_332(a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,TRANSPOSE(b))
        END SUBROUTINE zgemm_332
        END MODULE zgemm_32

        MODULE zgemm_33
        CONTAINS
        SUBROUTINE zgemm_333(a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,CONJG(TRANSPOSE(b))) + beta * c
        END SUBROUTINE zgemm_333
        END MODULE zgemm_33

        MODULE zgemm_34
        CONTAINS
        SUBROUTINE zgemm_334(a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,CONJG(TRANSPOSE(b)))
        END SUBROUTINE zgemm_334
        END MODULE zgemm_34

        MODULE zgemm_35
        CONTAINS
        SUBROUTINE zgemm_335(a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,b) + beta * c
        END SUBROUTINE zgemm_335
        END MODULE zgemm_35

        MODULE zgemm_36
        CONTAINS
        SUBROUTINE zgemm_336(a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:,:), b(:,:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,b)
        END SUBROUTINE zgemm_336
        END MODULE zgemm_36

        MODULE dgemm_01
        CONTAINS
        SUBROUTINE dgemm_301(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_301
        END MODULE dgemm_01

        MODULE dgemm_02
        CONTAINS
        SUBROUTINE dgemm_302(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_302
        END MODULE dgemm_02

        MODULE dgemm_03
        CONTAINS
        SUBROUTINE dgemm_303(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_303
        END MODULE dgemm_03

        MODULE dgemm_04
        CONTAINS
        SUBROUTINE dgemm_304(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_304
        END MODULE dgemm_04

        MODULE dgemm_05
        CONTAINS
        SUBROUTINE dgemm_305(alpha,op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),b) + beta * c
        END SUBROUTINE dgemm_305
        END MODULE dgemm_05

        MODULE dgemm_06
        CONTAINS
        SUBROUTINE dgemm_306(alpha,op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),b)
        END SUBROUTINE dgemm_306
        END MODULE dgemm_06

        MODULE dgemm_07
        CONTAINS
        SUBROUTINE dgemm_307(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_307
        END MODULE dgemm_07

        MODULE dgemm_08
        CONTAINS
        SUBROUTINE dgemm_308(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_308
        END MODULE dgemm_08

        MODULE dgemm_09
        CONTAINS
        SUBROUTINE dgemm_309(alpha,op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_309
        END MODULE dgemm_09

        MODULE dgemm_10
        CONTAINS
        SUBROUTINE dgemm_310(alpha,op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_310
        END MODULE dgemm_10

        MODULE dgemm_11
        CONTAINS
        SUBROUTINE dgemm_311(alpha,op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),b) + beta * c
        END SUBROUTINE dgemm_311
        END MODULE dgemm_11

        MODULE dgemm_12
        CONTAINS
        SUBROUTINE dgemm_312(alpha,op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(TRANSPOSE(a),b)
        END SUBROUTINE dgemm_312
        END MODULE dgemm_12

        MODULE dgemm_13
        CONTAINS
        SUBROUTINE dgemm_313(alpha,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_313
        END MODULE dgemm_13

        MODULE dgemm_14
        CONTAINS
        SUBROUTINE dgemm_314(alpha,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,TRANSPOSE(b))
        END SUBROUTINE dgemm_314
        END MODULE dgemm_14

        MODULE dgemm_15
        CONTAINS
        SUBROUTINE dgemm_315(alpha,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_315
        END MODULE dgemm_15

        MODULE dgemm_16
        CONTAINS
        SUBROUTINE dgemm_316(alpha,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,TRANSPOSE(b))
        END SUBROUTINE dgemm_316
        END MODULE dgemm_16

        MODULE dgemm_17
        CONTAINS
        SUBROUTINE dgemm_317(alpha,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,b) + beta * c
        END SUBROUTINE dgemm_317
        END MODULE dgemm_17

        MODULE dgemm_18
        CONTAINS
        SUBROUTINE dgemm_318(alpha,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = alpha * MATMUL(a,b)
        END SUBROUTINE dgemm_318
        END MODULE dgemm_18

        MODULE dgemm_19
        CONTAINS
        SUBROUTINE dgemm_319(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_319
        END MODULE dgemm_19

        MODULE dgemm_20
        CONTAINS
        SUBROUTINE dgemm_320(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_320
        END MODULE dgemm_20

        MODULE dgemm_21
        CONTAINS
        SUBROUTINE dgemm_321(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_321
        END MODULE dgemm_21

        MODULE dgemm_22
        CONTAINS
        SUBROUTINE dgemm_322(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_322
        END MODULE dgemm_22

        MODULE dgemm_23
        CONTAINS
        SUBROUTINE dgemm_323(op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),b) + beta * c
        END SUBROUTINE dgemm_323
        END MODULE dgemm_23

        MODULE dgemm_24
        CONTAINS
        SUBROUTINE dgemm_324(op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),b)
        END SUBROUTINE dgemm_324
        END MODULE dgemm_24

        MODULE dgemm_25
        CONTAINS
        SUBROUTINE dgemm_325(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_325
        END MODULE dgemm_25

        MODULE dgemm_26
        CONTAINS
        SUBROUTINE dgemm_326(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type, blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_326
        END MODULE dgemm_26

        MODULE dgemm_27
        CONTAINS
        SUBROUTINE dgemm_327(op_a,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_327
        END MODULE dgemm_27

        MODULE dgemm_28
        CONTAINS
        SUBROUTINE dgemm_328(op_a,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a, op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),TRANSPOSE(b))
        END SUBROUTINE dgemm_328
        END MODULE dgemm_28

        MODULE dgemm_29
        CONTAINS
        SUBROUTINE dgemm_329(op_a,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),b) + beta * c
        END SUBROUTINE dgemm_329
        END MODULE dgemm_29

        MODULE dgemm_30
        CONTAINS
        SUBROUTINE dgemm_330(op_a,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_a
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(TRANSPOSE(a),b)
        END SUBROUTINE dgemm_330
        END MODULE dgemm_30

        MODULE dgemm_31
        CONTAINS
        SUBROUTINE dgemm_331(a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_331
        END MODULE dgemm_31

        MODULE dgemm_32
        CONTAINS
        SUBROUTINE dgemm_332(a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,TRANSPOSE(b))
        END SUBROUTINE dgemm_332
        END MODULE dgemm_32

        MODULE dgemm_33
        CONTAINS
        SUBROUTINE dgemm_333(a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,TRANSPOSE(b)) + beta * c
        END SUBROUTINE dgemm_333
        END MODULE dgemm_33

        MODULE dgemm_34
        CONTAINS
        SUBROUTINE dgemm_334(a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,TRANSPOSE(b))
        END SUBROUTINE dgemm_334
        END MODULE dgemm_34

        MODULE dgemm_35
        CONTAINS
        SUBROUTINE dgemm_335(a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,b) + beta * c
        END SUBROUTINE dgemm_335
        END MODULE dgemm_35

        MODULE dgemm_36
        CONTAINS
        SUBROUTINE dgemm_336(a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:,:), b(:,:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          c = MATMUL(a,b)
        END SUBROUTINE dgemm_336
        END MODULE dgemm_36
