        !
        ! Replaces ZGERU   4 procedures
        ! Replaces ZGERC   4 procedures
        ! Replaces DGER    4 procedures
        ! 
        !
        ! Level 2 COMPLEX  (ZGERU, ZGERC)
        !
        !         alpha      op_b      beta     c
        !
        !  201    PRESENT              PRESENT   c < alpha a b(T) + beta c
        !  202    PRESENT                0.0     c < alpha a b(T) 
        !  203    PRESENT  Hermite     PRESENT   c < alpha a b(H) + beta c
        !  204    PRESENT  Hermite       0.0     c < alpha a b(H) 
        !  205      1.0                PRESENT   c < a b(T) + beta c
        !  206      1.0                  0.0     c < a b(T) 
        !  207      1.0    Hermite     PRESENT   c < a b(H) + beta c
        !  208      1.0    Hermite       0.0     c < a b(H) 
        !
        !
        MODULE zger_01
        CONTAINS
        SUBROUTINE zger_201(alpha,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = (alpha * b(i)) * a + beta * c(:,i)
          END DO
        END SUBROUTINE zger_201
        END MODULE zger_01
        !
        MODULE zger_02
        CONTAINS
        SUBROUTINE zger_202(alpha,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = (alpha * b(i)) * a
          END DO
        END SUBROUTINE zger_202
        END MODULE zger_02
        !
        MODULE zger_03
        CONTAINS
        SUBROUTINE zger_203(alpha,a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha, beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = (alpha * CONJG(b(i))) * a + beta * c(:,i)
          END DO
        END SUBROUTINE zger_203
        END MODULE zger_03
        !
        MODULE zger_04
        CONTAINS
        SUBROUTINE zger_204(alpha,a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: alpha
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = (alpha * CONJG(b(i))) * a
          END DO
        END SUBROUTINE zger_204
        END MODULE zger_04
        !
        MODULE zger_05
        CONTAINS
        SUBROUTINE zger_205(a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = b(i) * a + beta * c(:,i)
          END DO
        END SUBROUTINE zger_205
        END MODULE zger_05
        !
        MODULE zger_06
        CONTAINS
        SUBROUTINE zger_206(a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = b(i) * a
          END DO
        END SUBROUTINE zger_206
        END MODULE zger_06
        !
        MODULE zger_07
        CONTAINS
        SUBROUTINE zger_207(a,op_b,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          COMPLEX (wp), INTENT (IN) :: beta
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = CONJG(b(i)) * a + beta * c(:,i)
          END DO
        END SUBROUTINE zger_207
        END MODULE zger_07
        !
        MODULE zger_08
        CONTAINS
        SUBROUTINE zger_208(a,op_b,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          USE blas_keywords, ONLY : blas_conj_trans_type
          ! .. Scalar Arguments ..
          TYPE (blas_conj_trans_type), INTENT (IN) :: op_b
          ! .. Array Arguments ..
          COMPLEX (wp), INTENT (IN) :: a(:), b(:)
          COMPLEX (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = CONJG(b(i)) * a
          END DO
        END SUBROUTINE zger_208
        END MODULE zger_08
        !
        !
        ! Level 2 REAL   (DGER)
        !
        !         alpha      beta     c
        !
        !  201    PRESENT   PRESENT   c < alpha a b(T) + beta c
        !  202    PRESENT     0.0     c < alpha a b(T) 
        !  203      1.0     PRESENT   c < a b(T) + beta c
        !  204      1.0       0.0     c < a b(T) 
        !
        !
        MODULE dger_01
        CONTAINS
        SUBROUTINE dger_201(alpha,a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha, beta
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:), b(:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = (alpha * b(i)) * a + beta * c(:,i)
          END DO
        END SUBROUTINE dger_201
        END MODULE dger_01
        !
        MODULE dger_02
        CONTAINS
        SUBROUTINE dger_202(alpha,a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: alpha
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:), b(:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = (alpha * b(i)) * a
          END DO
        END SUBROUTINE dger_202
        END MODULE dger_02
        !
        MODULE dger_03
        CONTAINS
        SUBROUTINE dger_203(a,b,beta,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          REAL (wp), INTENT (IN) :: beta
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:), b(:)
          REAL (wp), INTENT (INOUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = b(i) * a + beta * c(:,i)
          END DO
        END SUBROUTINE dger_203
        END MODULE dger_03
        !
        MODULE dger_04
        CONTAINS
        SUBROUTINE dger_204(a,b,c)
          ! .. Use Statements ..
          USE blas_kinds, ONLY : wp => dp
          ! .. Scalar Arguments ..
          ! .. Array Arguments ..
          REAL (wp), INTENT (IN) :: a(:), b(:)
          REAL (wp), INTENT (OUT) :: c(:,:)
          ! temp body
          DO i = 1, SIZE(B)
            c(:,i) = b(i) * a
          END DO
        END SUBROUTINE dger_204
        END MODULE dger_04
